︠6bcf4628-ce95-4880-80dd-eb960191a6ads︠
class singlevertex(object):

    # variables for specifying arbitrary labeled singlevertex with arbitrary angle units, i.e. degrees/radians/other

    # 2n-tuple of positive real numbers with equal alternate sums
    Kpoint = []
    # 2n-tuple of real numbers beginning with 0, alternating up/down, last term > 0
    shadow = []

    alternate_sum = 0
    total_sum = 0

    # number of creases emanating from central vertex
    degree = 0
    # half the vertex degree
    n = 0

    # unit conversions
    intr2deg = 1
    deg2intr = 1
    intr2rad = 1
    rad2intr = 1
    deg2rad = pi/180
    rad2deg = 180/pi
    Kpoint_deg = []
    shadow_deg = []
    Kpoint_rad = []
    shadow_rad = []

    # stacking
    stacking = []
    stack_diagram = []
    Vpoint = []

    # embedding in the plane
    isolation_radius = 0
    layer_height = 0
    point_count_2d = 0
    points_2d = []
    polygon_2d = []
    plot_2d = []
    delta_steps = 10
    delta_step = 1.0
    animate_2d = 0

    # embedding in 3-space
    point_count_3d = 0
    polygon_3d = []
    # (x,y,z) position of central vertex
    central_vertex = []


    def __repr__(self):
        return "singlevertex"

    # ************
    #  horizontal
    # ************

    def set_Kpoint(self, v):
        # TODO: verify input
        self.Kpoint = v
        self.shadow = self.Kpoint2shadow(self.Kpoint)
        self.evaluate_metrics()

    def set_shadow(self, v):
        # TODO: verify input
        self.shadow = v
        self.Kpoint = self.shadow2Kpoint(self.shadow)
        self.evaluate_metrics()

    def Kpoint2shadow(self, v):
        shadow = [0]
        for i in range(len(v) - 1):
            sign = 1 - (i % 2) * 2
            shadow.append(shadow[-1] + sign * v[i])
        return shadow

    def shadow2Kpoint(self, v):
        Kpoint = []
        for i in range(len(v) // 2):
            Kpoint.append(v[2 * i + 1] - v[2 * i])
            Kpoint.append(v[2 * i + 1] - v[(2 * i + 2) % len(v)])
        return Kpoint

    def evaluate_metrics(self):
        self.degree = len(self.Kpoint)
        self.n = self.degree // 2

        self.total_sum = sum(self.Kpoint)
        self.alternate_sum = self.total_sum / 2

        self.intr2deg = 180 / self.alternate_sum
        self.deg2intr = self.alternate_sum / 180
        self.intr2rad = pi / self.alternate_sum
        self.rad2intr = self.alternate_sum / pi

        self.Kpoint_deg = [x * self.intr2deg for x in self.Kpoint]
        self.shadow_deg = [x * self.intr2deg for x in self.shadow]
        self.Kpoint_rad = [x * self.intr2rad for x in self.Kpoint]
        self.shadow_rad = [x * self.intr2rad for x in self.shadow]

    # **********
    #  vertical
    # **********

    def set_stacking(self, v):
        self.stacking = v
        self.Vpoint = self.stacking2Vpoint(self.stacking)

    def stacking2Vpoint(self, v):
        Vpoint = []
        for i in range(len(v)):
            Vpoint.append(abs(v[(i+1)%len(v)] - v[i]))
        return Vpoint

    # ***************
    #  stack diagram
    # ***************



    # ******************
    #  planar embedding
    # ******************

    def construct_planar_embedding(self, color='black', thickness=3, delta_steps=10):
        self.delta_steps = delta_steps
        self.delta_step = 1.0 / (self.delta_steps - 1)
        self.polygon_2d = []
        self.plot_2d = []
        for i in range(delta_steps):
            self.compute_isolation_radius(delta=(i / (self.delta_steps - 1)))
            self.compute_layer_height()
            self.compute_points_2d()
            poly = polygon2d(self.points_2d, color=color, fill=False, thickness=thickness)
            self.polygon_2d.append(poly)
            self.plot_2d.append(plot(poly))
        self.animate_2d = animate(self.plot_2d + self.plot_2d[::-1])

    def compute_isolation_radius(self, delta=1.0):
        self.isolation_radius = abs(self.shadow[0] - self.shadow[1])
        for i in range(self.degree):
            for j in range(self.degree):
                diff = self.shadow[i] - self.shadow[j]
                if i != j and self.parity(i) * diff > 0 and abs(diff) < self.isolation_radius:
                    self.isolation_radius = abs(diff)
        self.isolation_radius /= 3
        self.isolation_radius *= delta

    def compute_layer_height(self):
        self.layer_height = self.isolation_radius * sqrt(2) / (self.degree - 1.0)

    def compute_points_2d(self):
        self.point_count_2d = self.degree * 3
        self.points_2d = []
        for i in range(self.degree):
            i1 = (i + 1) % self.degree
            ipar = self.parity(i)
            x = self.shadow[i] + (ipar * self.Vpoint[i] * self.isolation_radius) / (self.degree - 1)
            y = self.stacking[i] * self.layer_height
            self.points_2d.append((x,y))
            x = self.shadow[i1] - (ipar * self.Vpoint[i1] * self.isolation_radius) / (self.degree - 1)
            y = self.stacking[i] * self.layer_height
            self.points_2d.append((x,y))
            x = self.shadow[i1] - (ipar * (2 - sqrt(2)) * self.Vpoint[i1] * self.isolation_radius) / (2 * self.degree - 2)
            y = 0.5 * (self.stacking[i] + self.stacking[i1]) * self.layer_height
            self.points_2d.append((x,y))


    # arithmetic tools

    # subtract 1 modulo n within [1,...,n]
    def mod_decrement(self, i,n):
        return (i+n-2)%n+1

    # add 1 modulo n within [1,...,n]
    def mod_increment(self, i,n):
        return (i%n)+1

    # 1 if even, -1 if odd
    def parity(self, i):
        return 1 if i % 2 == 0 else -1

    def get_polygons(self):
        return 0
︡aaeba84c-6832-4543-a560-4ba4df695e13︡{"done":true}︡
︠fd8ca8fc-1f9f-4f9c-9ede-8bb51b6a0364s︠

A = singlevertex()
k = [1,2,3,3,2,1]
A.set_Kpoint(k)
print(A.shadow)
print(A.alternate_sum)
print(A.total_sum)
print(A.Kpoint_deg)
print(A.Kpoint_rad)

# s = [0, 3, -1, 5]
# A.set_shadow(s)
# print(A.Kpoint)
# print(A.alternate_sum)
# print(A.total_sum)
# print(A.Kpoint_deg)
# print(A.Kpoint_rad)

stacking = [4,5,6,1,2,3]
A.set_stacking(stacking)
print(A.stacking)
print(A.Vpoint)

A.construct_planar_embedding(delta_steps=10)

A.animate_2d.show(gif=True)
# @interact
# def diagram(step=(0.0, 1.0, A.delta_step)):
#     show(A.polygon_2d[int(step / A.delta_step)])
︡f5c593b3-2a40-4e6e-831a-f624ac535745︡{"stdout":"[0, 1, -1, 2, -1, 1]\n"}︡{"stdout":"6\n"}︡{"stdout":"12\n"}︡{"stdout":"[30, 60, 90, 90, 60, 30]\n"}︡{"stdout":"[1/6*pi, 1/3*pi, 1/2*pi, 1/2*pi, 1/3*pi, 1/6*pi]\n"}︡{"stdout":"[4, 5, 6, 1, 2, 3]\n"}︡{"stdout":"[1, 1, 5, 1, 1, 1]\n"}︡{"file":{"filename":"/home/user/.sage/temp/project-904e15ae-7cf1-43f1-888e-5a68d75a7a69/131/tmp_vLzCnP.gif","show":true,"text":null,"uuid":"7ccde8dd-5fce-4f89-91e9-489b39fbd562"},"once":false}︡{"done":true}︡
︠6ca7bcc8-20f3-4206-99ee-3f36dc93ec80s︠


@interact
def func(k = slider(0, 10, .1), j = (-10, 10), l = range_slider(-5, 5)):
    print("k: %s" % k)
    print("j: %s" % j)
    print("l: [%s, %s]" % l)
︡00224f09-a5e5-4cd6-94db-ef822b9fb31d︡{"interact":{"controls":[{"animate":true,"control_type":"slider","default":0,"display_value":true,"label":"k","vals":["0.000000000000000","0.100000000000000","0.200000000000000","0.300000000000000","0.400000000000000","0.500000000000000","0.600000000000000","0.700000000000000","0.800000000000000","0.900000000000000","1.00000000000000","1.10000000000000","1.20000000000000","1.30000000000000","1.40000000000000","1.50000000000000","1.60000000000000","1.70000000000000","1.80000000000000","1.90000000000000","2.00000000000000","2.10000000000000","2.20000000000000","2.30000000000000","2.40000000000000","2.50000000000000","2.60000000000000","2.70000000000000","2.80000000000000","2.90000000000000","3.00000000000000","3.10000000000000","3.20000000000000","3.30000000000000","3.40000000000000","3.50000000000000","3.60000000000000","3.70000000000000","3.80000000000000","3.90000000000000","4.00000000000000","4.10000000000000","4.20000000000000","4.30000000000000","4.40000000000000","4.50000000000000","4.60000000000000","4.70000000000000","4.80000000000000","4.90000000000000","5.00000000000000","5.10000000000000","5.20000000000000","5.30000000000000","5.40000000000000","5.50000000000000","5.60000000000000","5.70000000000000","5.80000000000000","5.90000000000000","5.99999999999999","6.09999999999999","6.19999999999999","6.29999999999999","6.39999999999999","6.49999999999999","6.59999999999999","6.69999999999999","6.79999999999999","6.89999999999999","6.99999999999999","7.09999999999999","7.19999999999999","7.29999999999999","7.39999999999999","7.49999999999999","7.59999999999999","7.69999999999999","7.79999999999999","7.89999999999999","7.99999999999999","8.09999999999999","8.19999999999999","8.29999999999999","8.39999999999999","8.49999999999999","8.59999999999999","8.69999999999999","8.79999999999998","8.89999999999998","8.99999999999998","9.09999999999998","9.19999999999998","9.29999999999998","9.39999999999998","9.49999999999998","9.59999999999998","9.69999999999998","9.79999999999998","9.89999999999998","10.0000000000000"],"var":"k","width":null},{"animate":true,"control_type":"slider","default":0,"display_value":true,"label":"j","vals":["-10.0","-9.96","-9.92","-9.88","-9.84","-9.8","-9.76","-9.72","-9.68","-9.64","-9.6","-9.56","-9.52","-9.48","-9.44","-9.4","-9.36","-9.32","-9.28","-9.24","-9.2","-9.16","-9.12","-9.08","-9.04","-9.0","-8.96","-8.92","-8.88","-8.84","-8.8","-8.76","-8.72","-8.68","-8.64","-8.6","-8.56","-8.52","-8.48","-8.44","-8.4","-8.36","-8.32","-8.28","-8.24","-8.2","-8.16","-8.12","-8.08","-8.04","-8.0","-7.96","-7.92","-7.88","-7.84","-7.8","-7.76","-7.72","-7.68","-7.64","-7.6","-7.56","-7.52","-7.48","-7.44","-7.4","-7.36","-7.32","-7.28","-7.24","-7.2","-7.16","-7.12","-7.08","-7.04","-7.0","-6.96","-6.92","-6.88","-6.84","-6.8","-6.76","-6.72","-6.68","-6.64","-6.6","-6.56","-6.52","-6.48","-6.44","-6.4","-6.36","-6.32","-6.28","-6.24","-6.2","-6.16","-6.12","-6.08","-6.04","-6.0","-5.96","-5.92","-5.88","-5.84","-5.8","-5.76","-5.72","-5.68","-5.64","-5.6","-5.56","-5.52","-5.48","-5.44","-5.4","-5.36","-5.32","-5.28","-5.24","-5.2","-5.16","-5.12","-5.08","-5.04","-5.0","-4.96","-4.92","-4.88","-4.84","-4.8","-4.76","-4.72","-4.68","-4.64","-4.6","-4.56","-4.52","-4.48","-4.44","-4.4","-4.36","-4.32","-4.28","-4.24","-4.2","-4.16","-4.12","-4.08","-4.04","-4.0","-3.96","-3.92","-3.88","-3.84","-3.8","-3.76","-3.72","-3.68","-3.64","-3.6","-3.56","-3.52","-3.48","-3.44","-3.4","-3.36","-3.32","-3.28","-3.24","-3.2","-3.16","-3.12","-3.08","-3.04","-3.0","-2.96","-2.92","-2.88","-2.84","-2.8","-2.76","-2.72","-2.68","-2.64","-2.6","-2.56","-2.52","-2.48","-2.44","-2.4","-2.36","-2.32","-2.28","-2.24","-2.2","-2.16","-2.12","-2.08","-2.04","-2.0","-1.96","-1.92","-1.88","-1.84","-1.8","-1.76","-1.72","-1.68","-1.64","-1.6","-1.56","-1.52","-1.48","-1.44","-1.4","-1.36","-1.32","-1.28","-1.24","-1.2","-1.16","-1.12","-1.08","-1.04","-1.0","-0.96","-0.92","-0.88","-0.84","-0.8","-0.76","-0.72","-0.68","-0.64","-0.6","-0.56","-0.52","-0.48","-0.44","-0.4","-0.36","-0.32","-0.28","-0.24","-0.2","-0.16","-0.12","-0.08","-0.04","-3.60683705125e-14","0.04","0.08","0.12","0.16","0.2","0.24","0.28","0.32","0.36","0.4","0.44","0.48","0.52","0.56","0.6","0.64","0.68","0.72","0.76","0.8","0.84","0.88","0.92","0.96","1.0","1.04","1.08","1.12","1.16","1.2","1.24","1.28","1.32","1.36","1.4","1.44","1.48","1.52","1.56","1.6","1.64","1.68","1.72","1.76","1.8","1.84","1.88","1.92","1.96","2.0","2.04","2.08","2.12","2.16","2.2","2.24","2.28","2.32","2.36","2.4","2.44","2.48","2.52","2.56","2.6","2.64","2.68","2.72","2.76","2.8","2.84","2.88","2.92","2.96","3.0","3.04","3.08","3.12","3.16","3.2","3.24","3.28","3.32","3.36","3.4","3.44","3.48","3.52","3.56","3.6","3.64","3.68","3.72","3.76","3.8","3.84","3.88","3.92","3.96","4.0","4.04","4.08","4.12","4.16","4.2","4.24","4.28","4.32","4.36","4.4","4.44","4.48","4.52","4.56","4.6","4.64","4.68","4.72","4.76","4.8","4.84","4.88","4.92","4.96","5.0","5.04","5.08","5.12","5.16","5.2","5.24","5.28","5.32","5.36","5.4","5.44","5.48","5.52","5.56","5.6","5.64","5.68","5.72","5.76","5.8","5.84","5.88","5.92","5.96","6.0","6.04","6.08","6.12","6.16","6.2","6.24","6.28","6.32","6.36","6.4","6.44","6.48","6.52","6.56","6.6","6.64","6.68","6.72","6.76","6.8","6.84","6.88","6.92","6.96","7.0","7.04","7.08","7.12","7.16","7.2","7.24","7.28","7.32","7.36","7.4","7.44","7.48","7.52","7.56","7.6","7.64","7.68","7.72","7.76","7.8","7.84","7.88","7.92","7.96","8.0","8.04","8.08","8.12","8.16","8.2","8.24","8.28","8.32","8.36","8.4","8.44","8.48","8.52","8.56","8.6","8.64","8.68","8.72","8.76","8.8","8.84","8.88","8.92","8.96","9.0","9.04","9.08","9.12","9.16","9.2","9.24","9.28","9.32","9.36","9.4","9.44","9.48","9.52","9.56","9.6","9.64","9.68","9.72","9.76","9.8","9.84","9.88","9.92","9.96","10.0"],"var":"j","width":null},{"animate":true,"control_type":"range-slider","default":[250,500],"display_value":true,"label":"l","vals":["-5.0","-4.98","-4.96","-4.94","-4.92","-4.9","-4.88","-4.86","-4.84","-4.82","-4.8","-4.78","-4.76","-4.74","-4.72","-4.7","-4.68","-4.66","-4.64","-4.62","-4.6","-4.58","-4.56","-4.54","-4.52","-4.5","-4.48","-4.46","-4.44","-4.42","-4.4","-4.38","-4.36","-4.34","-4.32","-4.3","-4.28","-4.26","-4.24","-4.22","-4.2","-4.18","-4.16","-4.14","-4.12","-4.1","-4.08","-4.06","-4.04","-4.02","-4.0","-3.98","-3.96","-3.94","-3.92","-3.9","-3.88","-3.86","-3.84","-3.82","-3.8","-3.78","-3.76","-3.74","-3.72","-3.7","-3.68","-3.66","-3.64","-3.62","-3.6","-3.58","-3.56","-3.54","-3.52","-3.5","-3.48","-3.46","-3.44","-3.42","-3.4","-3.38","-3.36","-3.34","-3.32","-3.3","-3.28","-3.26","-3.24","-3.22","-3.2","-3.18","-3.16","-3.14","-3.12","-3.1","-3.08","-3.06","-3.04","-3.02","-3.0","-2.98","-2.96","-2.94","-2.92","-2.9","-2.88","-2.86","-2.84","-2.82","-2.8","-2.78","-2.76","-2.74","-2.72","-2.7","-2.68","-2.66","-2.64","-2.62","-2.6","-2.58","-2.56","-2.54","-2.52","-2.5","-2.48","-2.46","-2.44","-2.42","-2.4","-2.38","-2.36","-2.34","-2.32","-2.3","-2.28","-2.26","-2.24","-2.22","-2.2","-2.18","-2.16","-2.14","-2.12","-2.1","-2.08","-2.06","-2.04","-2.02","-2.0","-1.98","-1.96","-1.94","-1.92","-1.9","-1.88","-1.86","-1.84","-1.82","-1.8","-1.78","-1.76","-1.74","-1.72","-1.7","-1.68","-1.66","-1.64","-1.62","-1.6","-1.58","-1.56","-1.54","-1.52","-1.5","-1.48","-1.46","-1.44","-1.42","-1.4","-1.38","-1.36","-1.34","-1.32","-1.3","-1.28","-1.26","-1.24","-1.22","-1.2","-1.18","-1.16","-1.14","-1.12","-1.1","-1.08","-1.06","-1.04","-1.02","-1.0","-0.98","-0.96","-0.94","-0.92","-0.9","-0.88","-0.86","-0.84","-0.82","-0.8","-0.78","-0.76","-0.74","-0.72","-0.7","-0.68","-0.66","-0.64","-0.62","-0.6","-0.58","-0.56","-0.54","-0.52","-0.5","-0.48","-0.46","-0.44","-0.42","-0.4","-0.38","-0.36","-0.34","-0.32","-0.3","-0.28","-0.26","-0.24","-0.22","-0.2","-0.18","-0.16","-0.14","-0.12","-0.1","-0.08","-0.06","-0.04","-0.02","-1.80341852563e-14","0.02","0.04","0.06","0.08","0.1","0.12","0.14","0.16","0.18","0.2","0.22","0.24","0.26","0.28","0.3","0.32","0.34","0.36","0.38","0.4","0.42","0.44","0.46","0.48","0.5","0.52","0.54","0.56","0.58","0.6","0.62","0.64","0.66","0.68","0.7","0.72","0.74","0.76","0.78","0.8","0.82","0.84","0.86","0.88","0.9","0.92","0.94","0.96","0.98","1.0","1.02","1.04","1.06","1.08","1.1","1.12","1.14","1.16","1.18","1.2","1.22","1.24","1.26","1.28","1.3","1.32","1.34","1.36","1.38","1.4","1.42","1.44","1.46","1.48","1.5","1.52","1.54","1.56","1.58","1.6","1.62","1.64","1.66","1.68","1.7","1.72","1.74","1.76","1.78","1.8","1.82","1.84","1.86","1.88","1.9","1.92","1.94","1.96","1.98","2.0","2.02","2.04","2.06","2.08","2.1","2.12","2.14","2.16","2.18","2.2","2.22","2.24","2.26","2.28","2.3","2.32","2.34","2.36","2.38","2.4","2.42","2.44","2.46","2.48","2.5","2.52","2.54","2.56","2.58","2.6","2.62","2.64","2.66","2.68","2.7","2.72","2.74","2.76","2.78","2.8","2.82","2.84","2.86","2.88","2.9","2.92","2.94","2.96","2.98","3.0","3.02","3.04","3.06","3.08","3.1","3.12","3.14","3.16","3.18","3.2","3.22","3.24","3.26","3.28","3.3","3.32","3.34","3.36","3.38","3.4","3.42","3.44","3.46","3.48","3.5","3.52","3.54","3.56","3.58","3.6","3.62","3.64","3.66","3.68","3.7","3.72","3.74","3.76","3.78","3.8","3.82","3.84","3.86","3.88","3.9","3.92","3.94","3.96","3.98","4.0","4.02","4.04","4.06","4.08","4.1","4.12","4.14","4.16","4.18","4.2","4.22","4.24","4.26","4.28","4.3","4.32","4.34","4.36","4.38","4.4","4.42","4.44","4.46","4.48","4.5","4.52","4.54","4.56","4.58","4.6","4.62","4.64","4.66","4.68","4.7","4.72","4.74","4.76","4.78","4.8","4.82","4.84","4.86","4.88","4.9","4.92","4.94","4.96","4.98","5.0"],"var":"l","width":null}],"flicker":false,"id":"ccc93b69-fb44-4053-9e28-366ef2b5e0d5","layout":[[["k",12,null]],[["j",12,null]],[["l",12,null]],[["",12,null]]],"style":"None"}}︡{"done":true}︡
︠32c163e9-bcd1-474b-aee2-432d2b4e02bf︠









